@@ -17,11 +17,13 @@ from datetime import timedelta, datetime
 from importlib import import_module
 import re
 import os
+import pathlib
 from xml.sax.saxutils import quoteattr, escape
 
+from sphinx.errors import ConfigError, ExtensionError
 from sphinx.util.console import red
 from . import sphinx_compatibility, glr_path_static, __version__ as _sg_version
-from .utils import _replace_md5
+from .utils import _replace_md5, _has_optipng, _has_pypandoc
 from .backreferences import _finalize_backreferences
 from .gen_rst import (generate_dir_rst, SPHX_GLR_SIG, _get_memory_base,
                       _get_readme)
@@ -30,21 +32,33 @@ from .docs_resolv import embed_code_links
 from .downloads import generate_zipfiles
 from .sorting import NumberOfCodeLinesSortKey
 from .binder import copy_binder_files
+from .directives import MiniGallery
 
 
 _KNOWN_CSS = ('gallery', 'gallery-binder', 'gallery-dataframe')
 
+
+class DefaultResetArgv:
+    def __repr__(self):
+        return "DefaultResetArgv"
+
+    def __call__(self, gallery_conf, script_vars):
+        return []
+
+
 DEFAULT_GALLERY_CONF = {
     'filename_pattern': re.escape(os.sep) + 'plot',
     'ignore_pattern': r'__init__\.py',
     'examples_dirs': os.path.join('..', 'examples'),
+    'reset_argv': DefaultResetArgv(),
     'subsection_order': None,
     'within_subsection_order': NumberOfCodeLinesSortKey,
     'gallery_dirs': 'auto_examples',
     'backreferences_dir': None,
     'doc_module': (),
     'reference_url': {},
-    'capture_repr': ('_repr_html_','__repr__'),
+    'capture_repr': ('_repr_html_', '__repr__'),
+    'ignore_repr_types': r'',
     # Build options
     # -------------
     # We use a string for 'plot_gallery' rather than simply the Python boolean
@@ -61,16 +75,19 @@ DEFAULT_GALLERY_CONF = {
     'min_reported_time': 0,
     'binder': {},
     'image_scrapers': ('matplotlib',),
+    'compress_images': (),
     'reset_modules': ('matplotlib', 'seaborn'),
     'first_notebook_cell': '%matplotlib inline',
     'last_notebook_cell': None,
+    'notebook_images': False,
+    'pypandoc': False,
     'remove_config_comments': False,
     'show_memory': False,
     'junit': '',
     'log_level': {'backreference_missing': 'warning'},
     'inspect_global_variables': True,
-    'ignore_repr_types': r'',
     'css': _KNOWN_CSS,
+    'matplotlib_animations': False,
 }
 
 logger = sphinx_compatibility.getLogger('sphinx-gallery')
@@ -126,15 +143,50 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
             backreferences_warning,
             type=DeprecationWarning)
 
+    # Check capture_repr
+    capture_repr = gallery_conf['capture_repr']
+    supported_reprs = ['__repr__', '__str__', '_repr_html_']
+    if isinstance(capture_repr, tuple):
+        for rep in capture_repr:
+            if rep not in supported_reprs:
+                raise ConfigError("All entries in 'capture_repr' must be one "
+                                  "of %s, got: %s" % (supported_reprs, rep))
+    else:
+        raise ConfigError("'capture_repr' must be a tuple, got: %s"
+                          % (type(capture_repr),))
+    # Check ignore_repr_types
+    if not isinstance(gallery_conf['ignore_repr_types'], str):
+        raise ConfigError("'ignore_repr_types' must be a string, got: %s"
+                          % (type(gallery_conf['ignore_repr_types']),))
+
     # deal with show_memory
+    gallery_conf['memory_base'] = 0.
     if gallery_conf['show_memory']:
-        try:
-            from memory_profiler import memory_usage  # noqa, analysis:ignore
-        except ImportError:
-            logger.warning("Please install 'memory_profiler' to enable peak "
-                           "memory measurements.")
-            gallery_conf['show_memory'] = False
-    gallery_conf['memory_base'] = _get_memory_base(gallery_conf)
+        if not callable(gallery_conf['show_memory']):  # True-like
+            try:
+                from memory_profiler import memory_usage  # noqa
+            except ImportError:
+                logger.warning("Please install 'memory_profiler' to enable "
+                               "peak memory measurements.")
+                gallery_conf['show_memory'] = False
+            else:
+                def call_memory(func):
+                    mem, out = memory_usage(func, max_usage=True, retval=True,
+                                            multiprocess=True)
+                    try:
+                        mem = mem[0]  # old MP always returned a list
+                    except TypeError:  # 'float' object is not subscriptable
+                        pass
+                    return mem, out
+                gallery_conf['call_memory'] = call_memory
+                gallery_conf['memory_base'] = _get_memory_base(gallery_conf)
+        else:
+            gallery_conf['call_memory'] = gallery_conf['show_memory']
+    if not gallery_conf['show_memory']:  # can be set to False above
+        def call_memory(func):
+            return 0., func()
+        gallery_conf['call_memory'] = call_memory
+    assert callable(gallery_conf['call_memory'])
 
     # deal with scrapers
     scrapers = gallery_conf['image_scrapers']
@@ -152,11 +204,11 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
                     scraper = getattr(scraper, '_get_sg_image_scraper')
                     scraper = scraper()
                 except Exception as exp:
-                    raise ValueError('Unknown image scraper %r, got:\n%s'
-                                     % (orig_scraper, exp))
+                    raise ConfigError('Unknown image scraper %r, got:\n%s'
+                                      % (orig_scraper, exp))
             scrapers[si] = scraper
         if not callable(scraper):
-            raise ValueError('Scraper %r was not callable' % (scraper,))
+            raise ConfigError('Scraper %r was not callable' % (scraper,))
     gallery_conf['image_scrapers'] = tuple(scrapers)
     del scrapers
     # Here we try to set up matplotlib but don't raise an error,
@@ -173,6 +225,33 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     except (ImportError, ValueError):
         pass
 
+    # compress_images
+    compress_images = gallery_conf['compress_images']
+    if isinstance(compress_images, str):
+        compress_images = [compress_images]
+    elif not isinstance(compress_images, (tuple, list)):
+        raise ConfigError('compress_images must be a tuple, list, or str, '
+                          'got %s' % (type(compress_images),))
+    compress_images = list(compress_images)
+    allowed_values = ('images', 'thumbnails')
+    pops = list()
+    for ki, kind in enumerate(compress_images):
+        if kind not in allowed_values:
+            if kind.startswith('-'):
+                pops.append(ki)
+                continue
+            raise ConfigError('All entries in compress_images must be one of '
+                              '%s or a command-line switch starting with "-", '
+                              'got %r' % (allowed_values, kind))
+    compress_images_args = [compress_images.pop(p) for p in pops[::-1]]
+    if len(compress_images) and not _has_optipng():
+        logger.warning(
+            'optipng binaries not found, PNG %s will not be optimized'
+            % (' and '.join(compress_images),))
+        compress_images = ()
+    gallery_conf['compress_images'] = compress_images
+    gallery_conf['compress_images_args'] = compress_images_args
+
     # deal with resetters
     resetters = gallery_conf['reset_modules']
     if not isinstance(resetters, (tuple, list)):
@@ -181,12 +260,12 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     for ri, resetter in enumerate(resetters):
         if isinstance(resetter, str):
             if resetter not in _reset_dict:
-                raise ValueError('Unknown module resetter named %r'
-                                 % (resetter,))
+                raise ConfigError('Unknown module resetter named %r'
+                                  % (resetter,))
             resetters[ri] = _reset_dict[resetter]
         elif not callable(resetter):
-            raise ValueError('Module resetter %r was not callable'
-                             % (resetter,))
+            raise ConfigError('Module resetter %r was not callable'
+                              % (resetter,))
     gallery_conf['reset_modules'] = tuple(resetters)
 
     lang = lang if lang in ('python', 'python3', 'default') else 'python'
@@ -196,29 +275,60 @@ def _complete_gallery_conf(sphinx_gallery_conf, src_dir, plot_gallery,
     # Ensure the first cell text is a string if we have it
     first_cell = gallery_conf.get("first_notebook_cell")
     if (not isinstance(first_cell, str)) and (first_cell is not None):
-        raise ValueError("The 'first_notebook_cell' parameter must be type str"
-                         "or None, found type %s" % type(first_cell))
+        raise ConfigError("The 'first_notebook_cell' parameter must be type "
+                          "str or None, found type %s" % type(first_cell))
     # Ensure the last cell text is a string if we have it
     last_cell = gallery_conf.get("last_notebook_cell")
     if (not isinstance(last_cell, str)) and (last_cell is not None):
-        raise ValueError("The 'last_notebook_cell' parameter must be type str"
-                         "or None, found type %s" % type(last_cell))
+        raise ConfigError("The 'last_notebook_cell' parameter must be type str"
+                          " or None, found type %s" % type(last_cell))
+    # Check pypandoc
+    pypandoc = gallery_conf['pypandoc']
+    if not isinstance(pypandoc, (dict, bool)):
+        raise ConfigError("'pypandoc' parameter must be of type bool or dict,"
+                          "got: %s." % type(pypandoc))
+    gallery_conf['pypandoc'] = dict() if pypandoc is True else pypandoc
+    has_pypandoc, version = _has_pypandoc()
+    if isinstance(gallery_conf['pypandoc'], dict) and has_pypandoc is None:
+        logger.warning("'pypandoc' not available. Using Sphinx-Gallery to "
+                       "convert rst text blocks to markdown for .ipynb files.")
+        gallery_conf['pypandoc'] = False
+    elif isinstance(gallery_conf['pypandoc'], dict):
+        logger.info("Using pandoc version: %s to convert rst text blocks to "
+                    "markdown for .ipynb files" % (version,))
+    else:
+        logger.info("Using Sphinx-Gallery to convert rst text blocks to "
+                    "markdown for .ipynb files.")
+    if isinstance(pypandoc, dict):
+        accepted_keys = ('extra_args', 'filters')
+        for key in pypandoc:
+            if key not in accepted_keys:
+                raise ConfigError("'pypandoc' only accepts the following key "
+                                  "values: %s, got: %s."
+                                  % (accepted_keys, key))
+
     # Make it easy to know which builder we're in
     gallery_conf['builder_name'] = builder_name
     gallery_conf['titles'] = {}
-    # Ensure 'backreferences_dir' is str or None
+    # Ensure 'backreferences_dir' is str, pathlib.Path or None
     backref = gallery_conf['backreferences_dir']
-    if (not isinstance(backref, str)) and (backref is not None):
-        raise ValueError("The 'backreferences_dir' parameter must be of type"
-                         "str or None, found type %s" % type(backref))
+    if (not isinstance(backref, (str, pathlib.Path))) and \
+            (backref is not None):
+        raise ConfigError("The 'backreferences_dir' parameter must be of type "
+                          "str, pathlib.Path or None, "
+                          "found type %s" % type(backref))
+    # if 'backreferences_dir' is pathlib.Path, make str for Python <=3.5
+    # compatibility
+    if isinstance(backref, pathlib.Path):
+        gallery_conf['backreferences_dir'] = str(backref)
 
     if not isinstance(gallery_conf['css'], (list, tuple)):
-        raise TypeError('gallery_conf["css"] must be list or tuple, got %r'
-                        % (gallery_conf['css'],))
+        raise ConfigError('gallery_conf["css"] must be list or tuple, got %r'
+                          % (gallery_conf['css'],))
     for css in gallery_conf['css']:
         if css not in _KNOWN_CSS:
-            raise ValueError('Unknown css %r, must be one of %r'
-                             % (css, _KNOWN_CSS))
+            raise ConfigError('Unknown css %r, must be one of %r'
+                              % (css, _KNOWN_CSS))
         if gallery_conf['app'] is not None:  # can be None in testing
             gallery_conf['app'].add_css_file(css + '.css')
 
@@ -294,6 +404,7 @@ def generate_gallery_rst(app):
     examples_dirs = [ex_dir for ex_dir, _ in workdirs]
     files = collect_gallery_files(examples_dirs, gallery_conf)
     check_duplicate_filenames(files)
+    check_spaces_in_filenames(files)
 
     for examples_dir, gallery_dir in workdirs:
 
@@ -326,11 +437,12 @@ def generate_gallery_rst(app):
                 write_computation_times(gallery_conf, target_dir, this_costs)
 
             if gallery_conf['download_all_examples']:
-                download_fhindex = generate_zipfiles(gallery_dir)
+                download_fhindex = generate_zipfiles(
+                    gallery_dir, app.builder.srcdir)
                 fhindex.write(download_fhindex)
 
             fhindex.write(SPHX_GLR_SIG)
-        _replace_md5(index_rst_new)
+        _replace_md5(index_rst_new, mode='t')
     _finalize_backreferences(seen_backrefs, gallery_conf)
 
     if gallery_conf['plot_gallery']:
@@ -393,7 +505,8 @@ def _format_for_writing(costs, path, kind='rst'):
             t = '%0.2f sec' % (cost[0][0],)
         m = '{0:.1f} MB'.format(cost[0][1])
         lines.append([name, t, m])
-    lens = [max(x) for x in zip(*[[len(l) for l in ll] for ll in lines])]
+    lens = [max(x) for x in zip(*[[len(item) for item in cost]
+                                  for cost in lines])]
     return lines, lens
 
 
@@ -411,7 +524,7 @@ def write_computation_times(gallery_conf, target_dir, costs):
                   .format(_sec_to_readable(total_time), target_dir_clean))
         lines, lens = _format_for_writing(costs, target_dir_clean)
         del costs
-        hline = ''.join(('+' + '-' * (l + 2)) for l in lens) + '+\n'
+        hline = ''.join(('+' + '-' * (length + 2)) for length in lens) + '+\n'
         fid.write(hline)
         format_str = ''.join('| {%s} ' % (ii,)
                              for ii in range(len(lines[0]))) + '|\n'
@@ -578,9 +691,10 @@ def summarize_failing_examples(app, exception):
                 color='brown')
 
     if fail_msgs:
-        raise ValueError("Here is a summary of the problems encountered when "
-                         "running the examples\n\n" + "\n".join(fail_msgs) +
-                         "\n" + "-" * 79)
+        raise ExtensionError(
+            "Here is a summary of the problems encountered "
+            "when running the examples\n\n" + "\n".join(fail_msgs) +
+            "\n" + "-" * 79)
 
 
 def collect_gallery_files(examples_dirs, gallery_conf):
@@ -611,8 +725,20 @@ def check_duplicate_filenames(files):
 
     if len(dup_names) > 0:
         logger.warning(
-            'Duplicate file name(s) found. Having duplicate file names will '
-            'break some links. List of files: {}'.format(sorted(dup_names),))
+            'Duplicate example file name(s) found. Having duplicate file '
+            'names will break some links. '
+            'List of files: {}'.format(sorted(dup_names),))
+
+
+def check_spaces_in_filenames(files):
+    """Check for spaces in filenames across example directories."""
+    regex = re.compile(r'[\s]')
+    files_with_space = list(filter(regex.search, files))
+    if files_with_space:
+        logger.warning(
+            'Example file name(s) with space(s) found. Having space(s) in '
+            'file names will break some links. '
+            'List of files: {}'.format(sorted(files_with_space),))
 
 
 def get_default_config_value(key):
@@ -632,6 +758,9 @@ def setup(app):
     if 'sphinx.ext.autodoc' in app.extensions:
         app.connect('autodoc-process-docstring', touch_empty_backreferences)
 
+    # Add the custom directive
+    app.add_directive('minigallery', MiniGallery)
+
     app.connect('builder-inited', generate_gallery_rst)
     app.connect('build-finished', copy_binder_files)
     app.connect('build-finished', summarize_failing_examples)
