@@ -9,7 +9,6 @@ from __future__ import (division, absolute_import, print_function,
 import ast
 import codecs
 import importlib
-import io
 import tempfile
 import re
 import os
@@ -19,6 +18,7 @@ import codeop
 
 import pytest
 
+from sphinx.errors import ExtensionError
 import sphinx_gallery.gen_rst as sg
 from sphinx_gallery import downloads
 from sphinx_gallery.gen_gallery import generate_dir_rst
@@ -286,41 +286,52 @@ def test_extract_intro_and_title():
     assert intro_paragraph.replace('\n', ' ')[:95] == intro[:95]
 
     # Errors
-    with pytest.raises(ValueError, match='should have a header'):
+    with pytest.raises(ExtensionError, match='should have a header'):
         sg.extract_intro_and_title('<string>', '')  # no title
-    with pytest.raises(ValueError, match='Could not find a title'):
+    with pytest.raises(ExtensionError, match='Could not find a title'):
         sg.extract_intro_and_title('<string>', '=====')  # no real title
 
 
-def test_md5sums():
+@pytest.mark.parametrize(
+    'mode,expected_md5',
+    (['b', 'a546be453c8f436e744838a4801bd3a0'],
+     ['t', 'ea8a570e9f3afc0a7c3f2a17a48b8047']))
+def test_md5sums(mode, expected_md5):
     """Test md5sum check functions work on know file content."""
+    file_content = b'Local test\r\n'
     with tempfile.NamedTemporaryFile('wb', delete=False) as f:
-        f.write(b'Local test\n')
+        f.write(file_content)
     try:
-        file_md5 = sg.get_md5sum(f.name)
+        file_md5 = sg.get_md5sum(f.name, mode)
         # verify correct md5sum
-        assert 'ea8a570e9f3afc0a7c3f2a17a48b8047' == file_md5
+        assert file_md5 == expected_md5
         # False because is a new file
         assert not sg.md5sum_is_current(f.name)
         # Write md5sum to file to check is current
         with open(f.name + '.md5', 'w') as file_checksum:
             file_checksum.write(file_md5)
         try:
-            assert sg.md5sum_is_current(f.name)
+            assert sg.md5sum_is_current(f.name, mode)
         finally:
             os.remove(f.name + '.md5')
     finally:
         os.remove(f.name)
 
 
-def test_fail_example(gallery_conf, log_collector, req_pil):
+@pytest.mark.parametrize('failing_code, want', [
+    (CONTENT + ['#' * 79, 'First_test_fail', '#' * 79, 'second_fail'],
+     'not defined'),
+    (CONTENT + ['#' * 79, 'input("foo")', '#' * 79, 'second_fail'],
+     'Cannot use input'),
+    (CONTENT + ['#' * 79, 'bad syntax', '#' * 79, 'second_fail'],
+     'invalid syntax'),
+])
+def test_fail_example(gallery_conf, failing_code, want,
+                      log_collector, req_pil):
     """Test that failing examples are only executed until failing block."""
     gallery_conf.update(image_scrapers=(), reset_modules=())
     gallery_conf.update(filename_pattern='raise.py')
 
-    failing_code = CONTENT + ['#' * 79,
-                              'First_test_fail', '#' * 79, 'second_fail']
-
     with codecs.open(os.path.join(gallery_conf['examples_dir'], 'raise.py'),
                      mode='w', encoding='utf-8') as f:
         f.write('\n'.join(failing_code))
@@ -328,17 +339,22 @@ def test_fail_example(gallery_conf, log_collector, req_pil):
     sg.generate_file_rst('raise.py', gallery_conf['gallery_dir'],
                          gallery_conf['examples_dir'], gallery_conf)
     assert len(log_collector.calls['warning']) == 1
-    assert 'not defined' in log_collector.calls['warning'][0].args[2]
+    msg = log_collector.calls['warning'][0].args[2]
+    assert want in msg
+    assert 'gen_gallery' not in msg
+    # can only check that gen_rst is removed on non-input ones
+    if 'Cannot use input' not in msg:
+        assert 'gen_rst' not in msg
+    assert '_check_input' not in msg
 
     # read rst file and check if it contains traceback output
 
     with codecs.open(os.path.join(gallery_conf['gallery_dir'], 'raise.rst'),
                      mode='r', encoding='utf-8') as f:
         ex_failing_blocks = f.read().count('pytb')
-        if ex_failing_blocks == 0:
-            raise ValueError('Did not run into errors in bad code')
-        elif ex_failing_blocks > 1:
-            raise ValueError('Did not stop executing script after error')
+        assert ex_failing_blocks != 0, 'Did not run into errors in bad code'
+        assert ex_failing_blocks <= 1, \
+            'Did not stop executing script after error'
 
 
 def _generate_rst(gallery_conf, fname, content):
@@ -451,7 +467,7 @@ def test_gen_dir_rst(gallery_conf, fakesphinxapp, ext):
     args = (gallery_conf['src_dir'], gallery_conf['gallery_dir'],
             gallery_conf, [])
     if ext == '.bad':  # not found with correct ext
-        with pytest.raises(FileNotFoundError, match='does not have a README'):
+        with pytest.raises(ExtensionError, match='does not have a README'):
             generate_dir_rst(*args)
     else:
         out = generate_dir_rst(*args)
@@ -656,13 +672,11 @@ plt.close('all')
 fig
 """
 
-html_out = """.. only:: builder_html
+html_out = """.. raw:: html
 
-    .. raw:: html
-
-        <div> This is the _repr_html_ div </div>
-        <br />
-        <br />"""
+    <div> This is the _repr_html_ div </div>
+    <br />
+    <br />"""
 
 text_above_html = """Out:
 
@@ -677,7 +691,7 @@ text_above_html = """Out:
 def _clean_output(output):
     is_text = '.. rst-class:: sphx-glr-script-out' in output
 
-    is_html = '.. only:: builder_html' in output
+    is_html = '.. raw:: html' in output
 
     if output.isspace():
         return ''
@@ -748,10 +762,9 @@ def test_ignore_repr_types(gallery_conf, req_mpl, req_pil, script_vars):
 
 class TestLoggingTee:
     def setup(self):
-        self.output_file = io.StringIO()
         self.src_filename = 'source file name'
-        self.tee = sg.LoggingTee(self.output_file, sg.logger,
-                                 self.src_filename)
+        self.tee = sg._LoggingTee(self.src_filename)
+        self.output_file = self.tee.output
 
     def test_full_line(self, log_collector):
         # A full line is output immediately.
@@ -802,7 +815,7 @@ class TestLoggingTee:
     def test_isatty(self, monkeypatch):
         assert not self.tee.isatty()
 
-        monkeypatch.setattr(self.tee.output_file, 'isatty', lambda: True)
+        monkeypatch.setattr(self.tee.output, 'isatty', lambda: True)
         assert self.tee.isatty()
 
 
